@@ -3,11 +3,12 @@ package DateTime::Format::Natural;
 use strict;
 use warnings;
 use base qw(
-    DateTime::Format::Natural::Aliases
     DateTime::Format::Natural::Calc
     DateTime::Format::Natural::Duration
+    DateTime::Format::Natural::Extract
     DateTime::Format::Natural::Formatted
     DateTime::Format::Natural::Helpers
+    DateTime::Format::Natural::Rewrite
 );
 use boolean qw(true false);
 
@@ -18,7 +19,7 @@ use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
-our $VERSION = '0.88';
+our $VERSION = '0.96';
 
 validation_options(
     on_fail => sub
@@ -50,7 +51,7 @@ sub _init
     my %presets = (
         lang          => 'en',
         format        => 'd/m/y',
-        prefer_future => false,
+        prefer_future =>  false,
         time_zone     => 'floating',
     );
     foreach my $opt (keys %presets) {
@@ -63,7 +64,7 @@ sub _init
     }
     $self->{Daytime} = $opts{daytime} || {};
 
-    my $mod = __PACKAGE__.'::Lang::'.uc($self->{Lang});
+    my $mod = join '::', (__PACKAGE__, 'Lang', uc $self->{Lang});
     eval "require $mod"; die $@ if $@;
 
     $self->{data} = $mod->__new();
@@ -123,7 +124,7 @@ sub _init_vars
 {
     my $self = shift;
 
-    delete @$self{qw(modified postprocess)};
+    delete @$self{qw(keyword modified postprocess)};
 }
 
 sub parse_datetime
@@ -136,30 +137,24 @@ sub parse_datetime
 
     my $date_string = $self->{date_string};
 
-    $self->_rewrite_aliases(\$date_string);
-    $date_string =~ tr/,//d;
+    $self->_rewrite(\$date_string);
 
-    my ($formatted) = $date_string =~ m!^((?:\d+?[-./])+ (?:\d+?)) \b!x;
-
-    my %count;
-    if (defined $formatted) {
-        my @count = $formatted =~ m![-./]!g;
-        $count{$_}++ foreach @count;
-    }
+    my ($formatted) = $date_string =~ $self->{data}->__regexes('format');
+    my %count = $self->_count_separators($formatted);
 
     $self->{tokens} = [];
     $self->{traces} = [];
 
-    if (scalar keys %count == 1 && $count{(keys %count)[0]} == 2) {
+    if ($self->_check_formatted('ymd', \%count)) {
         my $dt = $self->_parse_formatted_ymd($date_string, \%count);
         return $dt if blessed($dt);
     }
-    elsif (scalar keys %count == 1 && $count{(keys %count)[0]} == 1 && (keys %count)[0] eq '/') {
+    elsif ($self->_check_formatted('md', \%count)) {
         my $dt = $self->_parse_formatted_md($date_string);
         return $dt if blessed($dt);
 
         if ($self->{Prefer_future}) {
-            $self->_advance_future(qw(md));
+            $self->_advance_future('md');
         }
     }
     elsif ($date_string =~ /^([+-]) (\d+?) ([a-zA-Z]+)$/x) {
@@ -264,6 +259,15 @@ sub parse_datetime_duration
       : do { $self->{duration} = false;
              ($duration_string) };
 
+    my $max = 2;
+
+    my $shrinked = false;
+    if (@date_strings > $max) {
+        my $offset = $max;
+        splice (@date_strings, $offset);
+        $shrinked = true;
+    }
+
     $self->_pre_duration(\@date_strings);
     $self->{state} = {};
 
@@ -280,7 +284,7 @@ sub parse_datetime_duration
         }
     }
 
-    $self->_post_duration(\@queue);
+    $self->_post_duration(\@queue, \@traces);
     $self->_restore_state;
 
     delete @$self{qw(duration insert state)};
@@ -288,9 +292,26 @@ sub parse_datetime_duration
     @{$self->{traces}} = @traces;
     $self->{input_string} = $duration_string;
 
+    if ($shrinked) {
+        $self->_set_failure;
+        $self->_set_error("(limit of $max duration substrings exceeded)");
+    }
+
     return @queue;
 }
 
+sub extract_datetime
+{
+    my $self = shift;
+
+    my $extract_string;
+    $self->_params_init(@_, { string => \$extract_string });
+
+    my @expressions = $self->_extract_expressions($extract_string);
+
+    return wantarray ? @expressions : $expressions[0];
+}
+
 sub success
 {
     my $self = shift;
@@ -337,7 +358,7 @@ sub _process
             my $valid_expression = true;
             my $definition = $expression->[0];
             my @positions = sort {$a <=> $b} keys %$definition;
-            my %regex_stack;
+            my (%first_stack, %rest_stack);
             foreach my $pos (@positions) {
                 if ($types->[$pos] eq 'SCALAR') {
                     if (defined $definition->{$pos}) {
@@ -351,9 +372,9 @@ sub _process
                     }
                 }
                 elsif ($types->[$pos] eq 'REGEXP') {
-                    local $1;
-                    if (${$self->_token($pos)} =~ $definition->{$pos}) {
-                        $regex_stack{$pos} = $1 if defined $1;
+                    if (my @captured = ${$self->_token($pos)} =~ $definition->{$pos}) {
+                        $first_stack{$pos} = shift @captured;
+                        $rest_stack{$pos} = [ @captured ];
                         next;
                     }
                     else {
@@ -371,7 +392,7 @@ sub _process
                 foreach my $check (@{$expression->[2]}) {
                     my @pos = @{$expression->[1][$i++]};
                     my $error;
-                    $valid_expression &= $check->(\%regex_stack, \@pos, \$error);
+                    $valid_expression &= $check->(\%first_stack, \%rest_stack, \@pos, \$error);
                     unless ($valid_expression) {
                         $self->_set_error("($error)");
                         last;
@@ -388,15 +409,16 @@ sub _process
                         $values[$c++] = ref $pos
                           ? $index eq 'VALUE'
                             ? $pos->{$index}
-                            : $self->SUPER::_helper($pos->{$index}, $regex_stack{$index})
-                          : exists $regex_stack{$index}
-                            ? $regex_stack{$index}
+                            : $self->SUPER::_helper($pos->{$index}, $first_stack{$index})
+                          : exists $first_stack{$index}
+                            ? $first_stack{$index}
                             : ${$self->_token($index)};
                     }
                     my $worker = "SUPER::$expression->[5]->[$i]";
                     $self->$worker(@values, $expression->[4]->[$i++]);
                 }
                 %opts = %{$expression->[6]};
+                $self->{keyword} = $keyword;
                 last PARSE;
             }
         }
@@ -411,7 +433,17 @@ sub _post_process
     my %opts = @_;
 
     if (exists $opts{truncate_to}) {
-        $self->{datetime}->truncate(to => $opts{truncate_to});
+        my @truncate_to = ref $opts{truncate_to} eq 'ARRAY' ? @{$opts{truncate_to}} : ($opts{truncate_to});
+        my $i = 0;
+        my @units = @{$self->{data}->__units('ordered')};
+        my %indexes = map { $_ => $i++ } @units;
+        foreach my $unit (@truncate_to) {
+            my $index = $indexes{$unit} - 1;
+            if (defined $units[$index] && !exists $self->{modified}{$units[$index]}) {
+                $self->{datetime}->truncate(to => $unit);
+                last;
+            }
+        }
     }
 
     if ($self->{Prefer_future} &&
@@ -434,21 +466,20 @@ sub _advance_future
           my $data = $_;
           any {
             my $token = $_;
-            $token =~ /$data/i;
+            $token =~ /^$data$/i;
           } @{$self->{tokens}}
         } @{$self->{data}->{$identifier}};
     };
 
     if ((all { /^(?:second|minute|hour)$/ } keys %modified)
         && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
-        && (exists $self->{modified}{minute} && $self->{modified}{minute} == 1)
         && $self->{datetime}->hour < DateTime->now(time_zone => $self->{Time_zone})->hour
     ) {
         $self->{postprocess}{day} = 1;
     }
     elsif ($token_contains->('weekdays_all')
         && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
-        && ($self->_Day_of_Week($self->{datetime}->year, $self->{datetime}->month, $self->{datetime}->day)
+        && ($self->_Day_of_Week(map $self->{datetime}->$_, qw(year month day))
          < DateTime->now(time_zone => $self->{Time_zone})->wday)
     ) {
         $self->{postprocess}{day} = 7;
@@ -541,6 +572,9 @@ DateTime::Format::Natural - Create machine readable date/time with natural parsi
 
  $parser = DateTime::Format::Natural->new;
 
+ $date_string  = $parser->extract_datetime($extract_string);
+ @date_strings = $parser->extract_datetime($extract_string);
+
  $dt = $parser->parse_datetime($date_string);
  @dt = $parser->parse_datetime_duration($date_string);
 
@@ -633,7 +667,7 @@ The date string.
 
 =head2 parse_datetime_duration
 
-Returns one or more L<DateTime> object(s) constructed from a human readable
+Returns one or two L<DateTime> objects constructed from a human readable
 date/time string which may contain timespans/durations. I<Same> interface
 and options as C<parse_datetime()>, but should be explicitly called in
 list context.
@@ -641,6 +675,20 @@ list context.
  @dt = $parser->parse_datetime_duration($date_string);
  @dt = $parser->parse_datetime_duration(string => $date_string);
 
+=head2 extract_datetime
+
+Returns parsable date/time substrings (also known as expressions) extracted
+from the string provided; in scalar context only the first parsable substring
+is returned, whereas in list context all parsable substrings are returned.
+Each extracted substring can then be passed to the C<parse_datetime()>/
+C<parse_datetime_duration()> methods.
+
+ $date_string  = $parser->extract_datetime($extract_string);
+ @date_strings = $parser->extract_datetime($extract_string);
+ # or
+ $date_string  = $parser->extract_datetime(string => $extract_string);
+ @date_strings = $parser->extract_datetime(string => $extract_string);
+
 =head2 success
 
 Returns a boolean indicating success or failure for parsing the date/time
@@ -652,9 +700,10 @@ Returns the error message if the parsing did not succeed.
 
 =head2 trace
 
-Returns one or more strings with traces of methods which were called within
-the Base class and a summary how often certain units have been modified.
-More than one string is commonly returned for durations.
+Returns one or two strings with the grammar keyword for the valid
+expression parsed, traces of methods which were called within the Calc
+class and a summary how often certain units have been modified. More than
+one string is commonly returned for durations. Useful as a debugging aid.
 
 =head1 GRAMMAR
 
@@ -666,12 +715,12 @@ you're intending to hack a bit on the grammar guts.
 
 =head1 EXAMPLES
 
-See the classes C<DateTime::Format::Natural::Lang::[language_code]> for a
+See the classes C<DateTime::Format::Natural::Lang::[language_code]> for an
 overview of currently valid input.
 
 =head1 BUGS & CAVEATS
 
-C<parse_datetime()>/C<parse_datetime_duration()> always return one or more
+C<parse_datetime()>/C<parse_datetime_duration()> always return one or two
 DateTime objects regardless whether the parse was successful or not. In
 case no valid expression was found or a failure occurred, an unaltered
 DateTime object with its initial values (most often the "current" now) is
@@ -715,6 +764,11 @@ valuable suggestions and patches:
  Eric Wilhelm
  Kevin Field
  Wes Morgan
+ Vladimir Marek
+ Rod Taylor
+ Tim Esselens
+ Colm Dougan
+ Chifung Fan
 
 =head1 SEE ALSO
 
